@@ -1,6 +1,6 @@
 package Catmandu::AlephX::Op::IllGetDocShort;
 use Catmandu::Sane;
-use Data::Util qw(:check :validate);
+use Catmandu::Util qw(:check :is);
 use Moo;
 
 with('Catmandu::AlephX::Response');
@@ -9,7 +9,7 @@ has z13 => (
   is => 'ro',
   lazy => 1,
   isa => sub{
-    hash_ref($_[0]);
+    check_hash_ref($_[0]);
   },
   default => sub { {}; }
 ); 
@@ -19,16 +19,19 @@ sub parse {
   my($class,$str_ref) = @_;
   my $xpath = xpath($str_ref);
 
+  my $op = op();
+
   my $z13 = {};
 
-  my($z) = $xpath->find('/ill-get-doc-short/z13')->get_nodelist();
+  my($z) = $xpath->find("/$op/z13")->get_nodelist();
 
   $z13 = get_children($z) if $z;
 
   __PACKAGE__->new(
-    session_id => $xpath->findvalue('/ill-get-doc-short/session-id'),
-    error => $xpath->findvalue('/ill-get-doc-short/error'),
-    z13 => $z13
+    session_id => $xpath->findvalue("/$op/session-id"),
+    errors => $class->parse_errors($xpath),
+    z13 => $z13,
+    content_ref => $str_ref
   );
 }
 
